
#include <kapp.h>
#include "kollins.h"

#include <qmime.h>
#include <qimage.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qsimplerichtext.h>
#include <qtooltip.h>
#include <qpixmap.h>
#include <qtextcodec.h>

#include <klocale.h>
#include <kaccel.h>
#include <kedittoolbar.h>
#include <kkeydialog.h>
#include <kmessagebox.h>
#include <kstddirs.h>

#include "fonetic.h"
#include <stdlib.h>

#define CAPTION_TABLE  {"Pl -> De","De -> Pl","Pl -> En","En -> Pl"}

Kollins::Kollins()
    : KMainWindow( 0, "kollins" )
{
    // set the shell's ui resource file
    setXMLFile("kollinsui.rc");

    config = new kydpConfig;
    config->load();

    this->setGeometry(config->geometry);

    splitterH = new QSplitter(this, "splitterH");
    dictList = new KListBox( splitterH, "dictList" );
    RTFOutput = new QTextBrowser (splitterH, "RTFOutput");
    setCentralWidget(splitterH);

    RTFOutput->setTextFormat( RichText );
    RTFOutput->setReadOnly(TRUE);
    RTFOutput->setLinkUnderline ( FALSE);
    dictList->setBottomScrollBar(FALSE);

    cb =  QApplication::clipboard();

    t = new DynamicTip( this );

    myDict = new ydpDictionary(config,dictList);

    menu = menuBar();
    mainToolBar = new KToolBar(this, "mainToolBar");

    m_popup = new QPopupMenu(this, "m_popup");

    InitActions();

    splitterH->setSizes(config->lst);

    m_checkTimer = new QTimer(this, "timer");
    m_checkTimer->start(1000, FALSE);

    dialogActiv = false;

    ksdock = new DockWidget(this);
    if(config->minimizeToTray)
    	ksdock->dock();

    ksdock->updateMenu(config->clipTracking);

    //tutaj jeszcze male zabezpieczenie - jesli user bedzie grzebac recznie w konfigu

    if(config->enableEnglishDict)
    	config->enableEnglishDict = myDict->CheckDictionary(ENGLISH_FILES);

    if(config->enableGermanDict)
	config->enableGermanDict = myDict->CheckDictionary(GERMAN_FILES);

    //
	while(!(config->enableEnglishDict || config->enableGermanDict)) {
		Configure(true);
		config->enableEnglishDict = myDict->CheckDictionary(ENGLISH_FILES);
		config->enableGermanDict = myDict->CheckDictionary(GERMAN_FILES);
	}

	if(!config->enableEnglishDict) config->useEnglish = false;
	if(!config->enableGermanDict) config->useEnglish = true;

	myDict->OpenDictionary();

    	toggleAction("dictionary_eng2pol") ->setEnabled(config->enableEnglishDict);
    	toggleAction("dictionary_pol2eng") ->setEnabled(config->enableEnglishDict);
        toggleAction("dictionary_ger2pol") ->setEnabled(config->enableGermanDict);
    	toggleAction("dictionary_pol2ger") ->setEnabled(config->enableGermanDict);

    UpdateLook();

    RTFOutput->mimeSourceFactory()->setFilePath( kapp->dirs()->findResourceDir("data", "kollins/tips.html" ) + "kollins/");
    RTFOutput->mimeSourceFactory()->setExtensionType("html", "text/html;charset=iso8859-2");

    setTabOrder( wordInput, dictList );

    connect( cb, SIGNAL(selectionChanged() ), SLOT(slotSelectionChanged()));
    connect( cb, SIGNAL( dataChanged() ), SLOT(slotClipboardChanged() ));
    connect(m_checkTimer, SIGNAL(timeout()), this, SLOT(newClipData()));
    connect (dictList, SIGNAL(highlighted(int)), this, SLOT(NewDefinition(int)));
    connect (dictList, SIGNAL(selected(int)), this, SLOT(PlayCurrent()));
    connect (wordInput, SIGNAL(textChanged(const QString&)), this, SLOT(NewFromLine(const QString&)));
    connect (wordInput, SIGNAL(returnPressed()), this, SLOT(PlayCurrent()));
    connect (RTFOutput, SIGNAL( highlighted ( const QString & )), this, SLOT(updateText( const QString & )));
    connect (dictList, SIGNAL(highlighted(const QString&)), this, SLOT(UpdateWinTitle(const QString&)));
}

void Kollins::test()
{

}

/* show event and hide event*/

void Kollins::showEvent(QShowEvent *e)
{
 if (config->minimizeToTray)
	show();
  else KMainWindow::showEvent(e);
}

void Kollins::hideEvent(QHideEvent *e)
{
if (config->minimizeToTray && !dialogActiv)
	hide();

  else KMainWindow::hideEvent(e);
}

void Kollins::clearInput()
{
wordInput->clear();
wordInput->setFocus();
}

void Kollins::updateGeometry()
{
config->geometry = QRect(pos().x(), pos().y(), size().width(), size().height());
config->lst = splitterH->sizes();
}

void Kollins::NewFromLine (const QString &newText)
{
	int index;
	QListBoxItem *result;

	if(!(result = dictList->findItem(newText, Qt::ExactMatch)))
		if(!(result = dictList->findItem(newText)))
			return;

	if ((index=dictList->index(result)) >= 0) {
		if (index!=dictList->currentItem())
			dictList->setCurrentItem(index);
		dictList->ensureCurrentVisible();
	}
}

/********* Tutaj kilka funkcji obslugujacych schowek *********/

void Kollins::clipboardSignalArrived( bool selectionMode )
{
	cb->setSelectionMode( selectionMode );
	QString text = cb->text();

	checkClipData( text );
}

void Kollins::checkClipData( const QString& text )
{
	static QString m_lastString;

	if (text != m_lastString) {
		m_lastString = text;
		if(config->clipTracking)
			PasteClippboard(m_lastString);
	}
}

void Kollins::newClipData()
{
	cb->setSelectionMode( true );
	QString clipContents = cb->text();

	checkClipData( clipContents );
}

/********* Funkcja inicjuje akcje i tworzy gui **********/

void Kollins::InitActions()
{
  KAction* action = KStdAction::print(this, SLOT(onPrint()), actionCollection());
  action -> setStatusText (i18n("Print definition"));
  action = KStdAction::quit (kapp, SLOT (quit()), actionCollection());
  action -> setStatusText (i18n("Quit Kollins"));

  KToggleAction* toggle  = new KToggleAction (i18n("&ENG -> POL"), "eng2pol", CTRL + Key_1, this, SLOT (SwapEngToPol()), \
  actionCollection(), "dictionary_eng2pol");
  toggle -> setStatusText (i18n("ENG -> POL"));
  toggle -> setExclusiveGroup("direction");
  toggle = new KToggleAction (i18n("POL -> ENG"), "pol2eng", CTRL + Key_2, this, SLOT (SwapPolToEng()), \
  actionCollection(), "dictionary_pol2eng");
  toggle -> setStatusText (i18n("POL -> ENG"));
  toggle -> setExclusiveGroup("direction");
  toggle = new KToggleAction (i18n("GER -> POL"), "ger2pol", CTRL + Key_1, this, SLOT (SwapGerToPol()), \
  actionCollection(), "dictionary_ger2pol");
  toggle -> setStatusText (i18n("GER -> POL"));
  toggle -> setExclusiveGroup("direction");
  toggle = new KToggleAction (i18n("POL -> GER"), "pol2ger", CTRL + Key_2, this, SLOT (SwapPolToGer()), \
  actionCollection(), "dictionary_pol2ger");
  toggle -> setStatusText (i18n("POL -> GER"));
  toggle -> setExclusiveGroup("direction");

  toggle = new KToggleAction(i18n("Clipboard Tracking"), "editpaste", CTRL + Key_P, this, SLOT (ClipboardTracking()), \
  actionCollection(), "clipboardTracking");
  toggle -> setStatusText (i18n("Clipboard Tracking"));

  action = KStdAction::showMenubar (this, SLOT (onViewMenubar()), actionCollection());
  action -> setStatusText (i18n("Shows/hides the menu bar"));
  action = KStdAction::showToolbar (this, SLOT (onViewToolbar()), actionCollection());
  action -> setStatusText (i18n("Shows/hides the tool bar"));
  action = KStdAction::configureToolbars(this, SLOT(ConfigureToolbars()), actionCollection());
  action -> setStatusText (i18n("Configure Toolbars"));
  action = KStdAction::keyBindings(this, SLOT(ConfigureShortcuts()), actionCollection());
  action -> setStatusText (i18n("Configure Shortcuts"));
  action = KStdAction::preferences(this, SLOT(ConfigureKollins()), actionCollection());
  action -> setStatusText (i18n("Configure Kollins"));

  action = new KAction (i18n("Clear input"), "locationbar_erase", Key_Return, this, SLOT(clearInput()), actionCollection(),
"clear_input");
  action = new KAction (i18n("Play Sample"), "player_play", CTRL + Key_N, this, SLOT(PlayCurrent()), actionCollection(),
"player_play");
action = new KAction (i18n("Reverse direction of translation"), "rotate", CTRL + Key_R, this, SLOT(reverseDirection()),
actionCollection(),"reverse_direction");
  action = new KWidgetAction (wordInput = new KLineEdit(), "Line Edit", CTRL + Key_A, this, SLOT(test()), actionCollection(), \
  "wordInput");

  createGUI();
}

/*******  Konfiguracja interfejsu *******/

void Kollins::ConfigureToolbars()
{
KEditToolbar dlg(actionCollection());
 if (dlg.exec())
 {
   createGUI();
 }
}

void Kollins::ConfigureShortcuts()
{
 KKeyDialog::configureKeys(actionCollection(),"kollinsui.rc");
}

/* Show and  hide toolbar */

void Kollins::onViewToolbar()
{
bool visible = toggleAction(KStdAction::stdName (KStdAction::ShowToolbar)) -> isChecked();
if (visible)
	toolBar("mainToolBar")->show();
else
	toolBar("mainToolBar")->hide();
}

void Kollins::onViewMenubar()
{
bool visible = toggleAction(KStdAction::stdName (KStdAction::ShowMenubar)) -> isChecked();
if (visible)
	menu->show();
else
	menu->hide();
}

/****** Funkcje zajmujace sie przelaczaniem jezykow *******/

void Kollins::SwapEngToPol ()
{
	SwapLang(1, 1);
}

void Kollins::SwapPolToEng ()
{
	SwapLang(0, 1);
}

void Kollins::SwapGerToPol ()
{
	SwapLang(1, 0);
}

void Kollins::SwapPolToGer ()
{
	SwapLang(0, 0);
}

void Kollins::SwapLang (bool direction, int lang ) //dir==1 toPolish
{
	int a;
	if(! ((config->toPolish == direction) && (config->useEnglish == lang)) ) {
		myDict->CloseDictionary();
		config->toPolish=direction;
		config->useEnglish = lang;
		do {
			a=myDict->OpenDictionary();
			if (a) Configure(TRUE);
		} while (a);
	}
	UpdateLook();
}

/******* Funkcja zajmuje sie wprowadzaniem zmian do wygladu interfejsu *******/

void Kollins::UpdateLook()
{
QBrush paper;
QPixmap pixmap;

if(QString::compare(config->kBckgrndPix, "NoBackground") != 0 && pixmap.load(config->kBckgrndPix)) {
	paper.setPixmap(pixmap);
	RTFOutput->setPaper( paper );
	dictList->setPaletteBackgroundPixmap( pixmap );
	dictList->setStaticBackground(TRUE);
}
else {
	RTFOutput->setPaper( white );
	dictList->setPaletteBackgroundColor(white);
}

dictList->setPaletteForegroundColor ( QColor(config->kFontKolor4) );

toggleAction("dictionary_eng2pol") -> setChecked(config->useEnglish && config->toPolish);
toggleAction("dictionary_pol2eng") -> setChecked(config->useEnglish && !config->toPolish);
toggleAction("dictionary_ger2pol") -> setChecked(!config->useEnglish && config->toPolish);
toggleAction("dictionary_pol2ger") -> setChecked(!config->useEnglish && !config->toPolish);

toggleAction("clipboardTracking") -> setChecked(config->clipTracking);

QString captions_tab[] = CAPTION_TABLE;
this -> setCaption(captions_tab[config->useEnglish*2+config->toPolish]);

UpdateFonts();
if(config->minimizeToTray)
	ksdock->dock();
else {
	this->show();
	ksdock->undock();
}

UpdateTip();
ksdock->updateMenu(config->clipTracking);

NewDefinition(dictList->currentItem() < 0 ? 0 : dictList->currentItem() );
}

/******* Ustawia kolor "czcionek fonetycznych" ******/

void Kollins::UpdateFonts()
{
QString tmp = ".\tc " + config->kFontKolor4;

char *tst;
(const char *)tst = tmp;
f1_xpm[2] = tst;
f2_xpm[2] = tst;
f3_xpm[2] = tst;
f4_xpm[2] = tst;
f5_xpm[2] = tst;
f6_xpm[2] = tst;
f7_xpm[2] = tst;
f8_xpm[2] = tst;

QMimeSourceFactory::defaultFactory()->setImage( "f1", QImage(f1_xpm) );
QMimeSourceFactory::defaultFactory()->setImage( "f2", QImage(f2_xpm) );
QMimeSourceFactory::defaultFactory()->setImage( "f3", QImage(f3_xpm) );
QMimeSourceFactory::defaultFactory()->setImage( "f4", QImage(f4_xpm) );
QMimeSourceFactory::defaultFactory()->setImage( "f5", QImage(f5_xpm) );
QMimeSourceFactory::defaultFactory()->setImage( "f6", QImage(f6_xpm) );
QMimeSourceFactory::defaultFactory()->setImage( "f7", QImage(f7_xpm) );
QMimeSourceFactory::defaultFactory()->setImage( "f8", QImage(f8_xpm) );
}

/********** Konfiguracja programu **********/

void Kollins::Configure(bool status)
{
	int result;
	dialogActiv = true;

	ydpConfigure *myConf = new ydpConfigure(config);

	m_checkTimer->stop();
	cb->blockSignals( TRUE );

	result = myConf->exec();

	if (!(result == QDialog::Accepted) && status ) {
		KMessageBox::sorry( this,
		i18n("Kollins can't work with incorrect path to dictionary files.\n"
		"Good bye!"), "kollins");
		if(myConf)
  			delete myConf;
		exit(0);
	}

	if(myConf)
        	delete myConf;

	if (result == QDialog::Accepted)	// just a performance gain
		config->save();
	cb->blockSignals( FALSE);
	m_checkTimer->start(1000, FALSE);
	dialogActiv= false;
}


void Kollins::ConfigureKollins()
{
	Configure(FALSE);
	UpdateLook();
}

/******* Ta funkcja zajmuje sie wyswietlaniem menu dla ikony w trayu *******/

void Kollins::contextMenu()
{
m_popup->exec(QCursor::pos());
}



void Kollins::ClipboardTracking()
{
config->clipTracking =  !config->clipTracking;
ksdock->updateMenu(config->clipTracking);
UpdateTip();
toggleAction("clipboardTracking") -> setChecked(config->clipTracking);
}

void Kollins::PasteClippboard(QString haslo)
{
	int index;
	QListBoxItem *result, *tmp_result;

	// do nothing if minimized
//	if (this->isMinimized())
//		return;

	QString contents = haslo.stripWhiteSpace();

	contents.truncate(20);

	if(!(result = dictList->findItem(contents, Qt::ExactMatch))) {
		int down = 0;
		int up = contents.length() + 1;

		while(up - down > 1) {
			if((tmp_result = dictList->findItem(contents.left((up+down)/2)))) {
				result = tmp_result; //tylko wtedy uaktualniaj gdy jest takie sowo
				down = (up + down)/2;
			}
			else
				up = (up + down)/2;
		}
	}

	if ((index=dictList->index(result)) >= 0) {
		if (index!=dictList->currentItem())
			dictList->setCurrentItem(index);
		dictList->ensureCurrentVisible();
		this->show();
		this->setActiveWindow ();
		this->raise();
	}

	wordInput->blockSignals( TRUE );
	wordInput->setText(contents);
	wordInput->blockSignals( FALSE );

	if(config->autoPlay)
		PlaySelected (index);
}

/******* Pobiera nowa definicje ze slownika *******/

void Kollins::NewDefinition (int index)
{
	QString def = myDict->GetDefinition(index);
	QTextCodec *codec = QTextCodec::codecForName("ISO8859-2");
        def = codec->toUnicode(def.fromAscii(def));
	RTFOutput->setText(def);
}

/***************** A teraz odgrywanie dzwieku  *******************/

void Kollins::PlaySelected (int index)
{
	if ( (index>=0) && config->toPolish )
	    myDict->Play(index,config);
}

void Kollins::PlayCurrent ()
{
	PlaySelected(dictList->currentItem());
}


/************* Tu zajmiemy sie wyswietlaniem podpowiedzi ***********/

void Kollins::updateText( const QString & href )
{

	static QString tab[] = {"", "adjective - przymiotnik", "adverb - przyswek", "conjunction - spjnik",
	"perfective verb - czasownik dokonany", "masculine(feminine) - mski(eski)", "inseparable verb - czasownik nierozdzielny",
	"invariable - niezmienny", "pronoun - zaimek", "neuter - nijaki", "plural noun - rzeczownik w liczbie mnogiej",
	"compound - wyraz zoony", "plural - liczba mnoga", "verb - czasownik", "intransitive verb - czasownik nieprzechodni",
	"transitive verb - czasownik przechodni", "singular - liczba pojedyncza",
	"abbreviation - skrt", "nominative - mianownik", "accusative - biernik", "dative - celownik",
	"genitive - dopeniacz", "infinitiv - bezokolicznik", "instrumental - narzdnik", "locative - miejscownik",
	"irregular - nieregularny", "preposition - przyimek", "auxiliary - pomocniczy", "past tense - czas przeszy",
	"past participle - imiesw czasu przeszego", "masculine - mski", "noun - rzeczownik", "feminine - eski",
	"postpositiv (does not immediately precede a noun) - nie wystpuje bezporednio przed rzeczownikiem",
	"nonvirile - niemskoosobowy", "virile - mskoosobowy", "number - liczba", "declined - odmienny", "exclemation - wykrzyknik",
	"offensiv - obraliwy, wulgarny", "informal - potocznie", "pejorative - pejoratywny", "compaund - wyraz zoony",
	"obraliwy - offensiv", "potoczny - informal", "reflexiv verb - czasownik zwrotny", "attribute - przydawka",
	"particle - partykua", "formal - formalny", "imperfect tens - czas przeszy o aspekcie niedokonanym",
	"diminutive - zdrobnienie", "administration - administracja",
	"anatomy - anatomia", "automobiles - motoryzacja", "aviation - lotnictwo",  "biology - biologia",
	"botany - botanika", "British English - angielszczyzna brytyjska", "chemistry - chemia", "commerce - handel",
	"computer - informatyka i komputery", "culinary - kulinarny", "electronics, eletricity - elektronika, elektryczno",
	"finance - finanse", "law - prawo", "linguistic - jzykoznawstwo", "mathematics - matematyka",
	"medicine - medycyna", "military - wojskowo", "music - muzyka", "nautical - egluga", "politics - polityka",
	"psychology - psychologia", "railways - kolej", "religion - religia", "school - szkoa", "sport - sportowy",
	"technology - technika i technologia", "telecominication - telekomunikacja", "theatre - teatr", "printing - poligrafia",
	"American English - angielszczyzna amerykaska", "zoology - zoologia", "figurative - przenony", "literal - dosowny",
	"geography - geografia", "architecture - architektura", "fizyka - physics", "physiology - fizjologia",
	"imperative - tryb rozkazujcy", "geology - geologia","article - rodzajnik", "indefinite - nieokrelony",
	"definite - okrelony", "photography - fotografia", "elektryczno - electricity", "ekonomia - economy",
	"economy - ekonomia", "geometria - geometry", "jzykoznawstwo - linguistic", "kulinarny - culinary",
	"komputery - computers", "lotnictwo - aviation", "matematyka - mathematics", "motoryzacja - automobiles",
	"muzyka - music", "szkoa - school", "wojskowo - military", "egluga - nautical", "budownictwo - construction",
	"meteorology - meteorologia", "history - historia", "poligrafia - printing", "rolnictwo - agriculture",
	"prefix - przedrostek", "astronomy - astronomia", "physics - fizyka", "et cetera - itd.", "agriculture - rolnictwo",
	"construction - budownictwo", "suffix - przyrostek", "literatura - literature", "university - uniwersytet", "?", "ups!"};

	QString tmp = href;

	tmp.remove(0, 10);

	if(tmp.toInt() >= 0 && tmp.toInt() < 124)
	t->tekst = tab[tmp.toInt()];
;

}

void Kollins::onPrint() //from qt examples
{
    KPrinter printer;
    printer.setFullPage(TRUE);
    if ( printer.setup( this ) ) {
        QPainter p( &printer );
        QPaintDeviceMetrics metrics(p.device());
        int dpix = metrics.logicalDpiX();
        int dpiy = metrics.logicalDpiY();
        const int margin = 72; // pt
        QRect body(margin*dpix/72, margin*dpiy/72,
                   metrics.width()-margin*dpix/72*2,
                   metrics.height()-margin*dpiy/72*2 );
        QSimpleRichText richText( RTFOutput->text(), QFont(), RTFOutput->context(), RTFOutput->styleSheet(),
                                  RTFOutput->mimeSourceFactory(), body.height() );
        richText.setWidth( &p, body.width() );
        QRect view( body );
        int page = 1;
        do {
            richText.draw( &p, body.left(), body.top(), view, colorGroup() );
            view.moveBy( 0, body.height() );
            p.translate( 0 , -body.height() );
            p.drawText( view.right() - p.fontMetrics().width( QString::number(page) ),
                        view.bottom() + p.fontMetrics().ascent() + 5, QString::number(page) );
            if ( view.top()  >= richText.height() )
                break;
            printer.newPage();
            page++;
        } while (TRUE);
    }
}

void Kollins::UpdateWinTitle(const QString& contents)
{
QString captions_tab[] = CAPTION_TABLE;
this -> setCaption(captions_tab[config->useEnglish*2+config->toPolish] + " - \""+ contents + "\"");
}

void Kollins::UpdateTip()
{
QString captions[] = CAPTION_TABLE;

QString tip = i18n("Translation: ") + captions[config->useEnglish*2+config->toPolish]
+ "\n" + i18n("Clipboard Tracking: ") + (config->clipTracking ? i18n("enabled") : i18n("disabled"));

QToolTip::add(ksdock, tip);
}

void Kollins::reverseDirection()
{
SwapLang(!config->toPolish, config->useEnglish);
}


/********* I wreszcie destruktor  *******/

Kollins::~Kollins()
{
myDict->CloseDictionary();
updateGeometry();
config->save();
delete t;
delete config;
delete m_checkTimer;
delete splitterH;
}

#include "kollins.moc"
